<?php

/**
 * @file
 * Contains \Drupal\custom_pub\Form\CustomPublishingOptionForm.
 */

namespace Drupal\custom_pub\Form;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Class CustomPublishingOptionForm.
 *
 * @package Drupal\custom_pub\Form
 */
class CustomPublishingOptionForm extends EntityForm {
  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $custom_publishing_option = $this->entity;

    $form['label'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $custom_publishing_option->label(),
      '#description' => $this->t("The label for this publishing option."),
      '#required' => TRUE,
    );

    $form['id'] = array(
      '#type' => 'machine_name',
      '#default_value' => $custom_publishing_option->id(),
      '#machine_name' => array(
        'exists' => '\Drupal\custom_pub\Entity\CustomPublishingOption::load',
      ),
      '#disabled' => !$custom_publishing_option->isNew(),
    );

    $form['description'] = array(
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#maxlength' => 255,
      '#default_value' => $custom_publishing_option->getDescription(),
      '#description' => $this->t("Add a description for this publishing option."),
      '#required' => TRUE,
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // check to see if a field definition exists by this machine name to prevent collisions/overwrites
    $entity = $this->entity;
    $manager = \Drupal::entityDefinitionUpdateManager();
    $storage_definition = $manager->getFieldStorageDefinition($entity->id(), 'node');

    if ($entity->isNew() && isset($storage_definition)) {
      $form_state->setError($form['id'], $this->t('Cannot use machine name %name - field definition already exists.', ['%name' => $entity->id()]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;

    $is_new = $entity->isNew();
    $id = $entity->id();
    $label = $entity->label();
    $description = $entity->getDescription();

    $status = $entity->save();
    $manager = \Drupal::entityDefinitionUpdateManager();

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label custom publishing option.', [
          '%label' => $label,
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label custom publishing option.', [
          '%label' => $label,
        ]));
    }

    if ($is_new) {
      // create the field definition for the node
      $storage_definition = BaseFieldDefinition::create('boolean')
        ->setLabel(t('!label', ['!label' => $label]))
        ->setDescription(t('!description', ['!description' => $description]))
        ->setRevisionable(TRUE)
        ->setTranslatable(TRUE)
        ->setDefaultValue(0)
        ->setDisplayConfigurable('form', TRUE);

      $manager->installFieldStorageDefinition($id, 'node', 'custom_pub', $storage_definition);
    } else {
      $storage_definition = $manager->getFieldStorageDefinition($id, 'node');

      if (isset($storage_definition)) {
        // update the label and description on the definition
        $storage_definition
          ->setLabel(t('!label', ['!label' => $label]))
          ->setDescription(t('!description', ['!description' => $description]));

        $manager->updateFieldStorageDefinition($storage_definition);
      }
    }

    $form_state->setRedirectUrl($entity->urlInfo('collection'));
  }

}
